﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
// I enable threading here but I am only using it from the SerialPortsLib because
// it can enable the connect button too soon, inadvertantly allow multiple clicks
using System.Threading;

// All serial port functions should be kept in a seperate project/lib so it can be
// used with other software and updated independantly without having to compile
// any projects that referance it unless it goes through a major change.
//
// NOTE: there is only two functions in the lib so far, so it don't do much.
//
//  For Visual C# 2008 Express users
//  Those of you new to .NET and C#, comment out the the following line
//
//      string[] currentPorts = ports.getPorts();
//
//  in the btnGetSerialPorts_Click method below, and retype it as if you wrote it.
//  You will notice a tool tip popup as you type ports.getPorts() with the XML
//  Comments from within the SerialPortsLib project.  View SerialPortsLib.cs to see
//  how to add these types of comments to make everyones experiance using the code
//  you write a bit more understandable without the need to read through several
//  lines of code to find and understand it.

namespace SerialPorts_Form
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            // this alters the title bar to display
            this.Text = Application.ProductName + " - Version " + Application.ProductVersion;

            // show some text in the textbox, Quotes need to be escaped
            txtInfo.Text = "Click on the \"Get Serial Ports\" button to populate the combo box with currently detected ports.";

            // Populate list boxes using the available enums, and select a default setting
            foreach (string s in Enum.GetNames(typeof(SerialPortsLib.SerialPortsLib.BaudRates)))
            {
                lstBaudRates.Items.Add(s);
            }
            lstBaudRates.SelectedIndex = 13;

            foreach (string s in Enum.GetNames(typeof(Parity)))
            {
                lstParity.Items.Add(s);
            }
            lstParity.SelectedIndex = 0;

            foreach (string s in Enum.GetNames(typeof(StopBits)))
            {
                lstStopBits.Items.Add(s);
            }
            lstStopBits.SelectedIndex = 2;

            foreach (string s in Enum.GetNames(typeof(Handshake)))
            {
                lstHandshake.Items.Add(s);
            }
            lstHandshake.SelectedIndex = 0;

        }

        private void btnGetSerialPorts_Click(object sender, EventArgs e)
        {
            // First, clear the combo box and any other data before proceding
            // This will prevent duplicate values when the button is clicked
            // more than once.
            cboPortList.Items.Clear();

            //Declare a new instance of the SerialPortsLib class and give it a simple name (ports)
            SerialPortsLib.SerialPortsLib ports = new SerialPortsLib.SerialPortsLib();

            // create a string array and populate it with the getPorts function above
            string[] currentPorts = ports.getPorts();

            // for each string contained in the array, add that string as an item in a
            // combo box called cboPortList
            foreach (string port in currentPorts)
            {
                cboPortList.Items.Add(port);
            }
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            //First, set the Connect button to disabled to prevent double clicks.
            btnConnect.Enabled = false;

            // break out of the connection if these values are null
            // and display a message to the user
            if (cboPortList.SelectedItem == null)
            {
                noPort();
                btnConnect.Enabled = true;
                return;
            }
            
            //Declare a new instance of the SerialPortsLib class and give it a simple
            //name (ports) 
            SerialPortsLib.SerialPortsLib ports = new SerialPortsLib.SerialPortsLib();
            // the above line automaticaly disposes the created ports referance when
            // the functon completes

            // the following comment is for referance of converting enums
            //
            //(System.IO.Ports.Handshake)Enum.Parse(typeof(System.IO.Ports.Handshake), Convert.ToString(lstHandshake.SelectedItem))

            string selected_Port;
            selected_Port = Convert.ToString(cboPortList.SelectedItem);

            SerialPortsLib.SerialPortsLib.BaudRates selected_Baudrate;
            selected_Baudrate = (SerialPortsLib.SerialPortsLib.BaudRates)Enum.Parse(typeof(SerialPortsLib.SerialPortsLib.BaudRates), Convert.ToString(lstBaudRates.SelectedItem));

            System.IO.Ports.Parity selected_Parity;
            selected_Parity = (Parity)Enum.Parse(typeof(Parity), Convert.ToString(lstParity.SelectedItem));

            int selected_DataBits;
            selected_DataBits = Convert.ToInt32(txtDataBits.Text);

            System.IO.Ports.StopBits selected_StopBits;
            selected_StopBits = (StopBits)Enum.Parse(typeof(StopBits), Convert.ToString(lstStopBits.SelectedItem));

            System.IO.Ports.Handshake selected_Handshake;
            selected_Handshake = (Handshake)Enum.Parse(typeof(Handshake), Convert.ToString(lstHandshake.SelectedItem));

            string selected_String;
            selected_String = txtStringToSend.Text;

            // call Heaters code
            ports.testSerialPort(selected_Port, selected_Baudrate, selected_Parity, selected_DataBits, selected_StopBits, selected_Handshake, selected_String);

            //re-enable the connect button
            btnConnect.Enabled = true;
        }

        private void noBaud()
        {
            MessageBox.Show("Please select a baud rate.");
        }

        private void noPort()
        {
            MessageBox.Show("Please select a port to connect to");
        }
        private void noParity()
        {
            MessageBox.Show("Please select parity");
        }
    }
}
